==> rxdart-0.22.6.tar.gz: lib/src/streams/repeat.dart <==
import 'dart:async';

/// Creates a [Stream] that will recreate and re-listen to the source
/// Stream the specified number of times until the [Stream] terminates
/// successfully.
///
/// If [count] is not specified, it repeats indefinitely.
///
/// ### Example
///
///     new RepeatStream((int repeatCount) =>
///       Observable.just('repeat index: $repeatCount'), 3)
///         .listen((i) => print(i)); // Prints 'repeat index: 0, repeat index: 1, repeat index: 2'
class RepeatStream<T> extends Stream<T> {
  /// The factory method used at subscription time
  final Stream<T> Function(int) streamFactory;

  /// The amount of repeat attempts that will be made
  /// If null or 0, then an indefinite amount of attempts will be made.
  final int count;
  int _repeatStep = 0;
  StreamController<T> _controller;
  StreamSubscription<T> _subscription;

  /// Constructs a [Stream] that will recreate and re-listen to the source
  /// [Stream] (created with the provided factory method).
  /// The count parameter specifies number of times the repeat will take place,
  /// until this [Stream] terminates successfully.
  /// If the count parameter is not specified, then this [Stream] will repeat
  /// indefinitely.
  RepeatStream(this.streamFactory, [this.count]);

  @override
  StreamSubscription<T> listen(
    void onData(T event), {
    Function onError,
    void onDone(),
    bool cancelOnError,
  }) {
    _controller ??= StreamController<T>(
        sync: true,
        onListen: _maybeRepeatNext,
        onPause: ([Future<dynamic> resumeSignal]) =>
            _subscription.pause(resumeSignal),
        onResume: () => _subscription.resume(),
        onCancel: () => _subscription?.cancel());

    return _controller.stream.listen(
      onData,
      onError: onError,
      onDone: onDone,
      cancelOnError: cancelOnError,
    );
  }

  void _repeatNext() {
    void onDone() {
      _subscription?.cancel();

      _maybeRepeatNext();
    }

    try {
      _subscription = streamFactory(_repeatStep++).listen(_controller.add,
          onError: _controller.addError, onDone: onDone, cancelOnError: false);
    } catch (e, s) {
      _controller.addError(e, s);
    }
  }

  void _maybeRepeatNext() {
    if (_repeatStep == count) {
      _controller.close();
    } else {
      _repeatNext();
    }
  }
}

==> rxdart-0.23.0-dev.1.tar.gz: lib/src/streams/repeat.dart <==
import 'dart:async';

/// Creates a [Stream] that will recreate and re-listen to the source
/// Stream the specified number of times until the [Stream] terminates
/// successfully.
///
/// If [count] is not specified, it repeats indefinitely.
///
/// ### Example
///
///     RepeatStream((int repeatCount) =>
///       Observable.just('repeat index: $repeatCount'), 3)
///         .listen((i) => print(i)); // Prints 'repeat index: 0, repeat index: 1, repeat index: 2'
class RepeatStream<T> extends Stream<T> {
  /// The factory method used at subscription time
  final Stream<T> Function(int) streamFactory;

  /// The amount of repeat attempts that will be made
  /// If null or 0, then an indefinite amount of attempts will be made.
  final int count;
  int _repeatStep = 0;
  StreamController<T> _controller;
  StreamSubscription<T> _subscription;

  /// Constructs a [Stream] that will recreate and re-listen to the source
  /// [Stream] (created with the provided factory method).
  /// The count parameter specifies number of times the repeat will take place,
  /// until this [Stream] terminates successfully.
  /// If the count parameter is not specified, then this [Stream] will repeat
  /// indefinitely.
  RepeatStream(this.streamFactory, [this.count]);

  @override
  StreamSubscription<T> listen(
    void onData(T event), {
    Function onError,
    void onDone(),
    bool cancelOnError,
  }) {
    _controller ??= StreamController<T>(
        sync: true,
        onListen: _maybeRepeatNext,
        onPause: ([Future<dynamic> resumeSignal]) =>
            _subscription.pause(resumeSignal),
        onResume: () => _subscription.resume(),
        onCancel: () => _subscription?.cancel());

    return _controller.stream.listen(
      onData,
      onError: onError,
      onDone: onDone,
      cancelOnError: cancelOnError,
    );
  }

  void _repeatNext() {
    void onDone() {
      _subscription?.cancel();

      _maybeRepeatNext();
    }

    try {
      _subscription = streamFactory(_repeatStep++).listen(_controller.add,
          onError: _controller.addError, onDone: onDone, cancelOnError: false);
    } catch (e, s) {
      _controller.addError(e, s);
    }
  }

  void _maybeRepeatNext() {
    if (_repeatStep == count) {
      _controller.close();
    } else {
      _repeatNext();
    }
  }
}
